@@ -14,7 +14,7 @@ from datetime import datetime
 
 from aimodelshare.aws import run_function_on_lambda, get_token, get_aws_token, get_aws_client
 
-from aimodelshare.aimsonnx import _get_leaderboard_data, inspect_model, _get_metadata, _model_summary, model_from_string
+from aimodelshare.aimsonnx import _get_leaderboard_data, inspect_model, _get_metadata, _model_summary, model_from_string, pyspark_model_from_string
 
 
 def _get_file_list(client, bucket, model_id):
@@ -208,7 +208,7 @@ def _update_leaderboard(
     # }}}
 
 def _update_leaderboard_public(
-    modelpath, eval_metrics, s3_presigned_dict, custom_metadata=None, private=False):
+    modelpath, eval_metrics, s3_presigned_dict, custom_metadata=None, private=False, leaderboard_type = "competition"):
 
     if private==True:
         mastertable_path = 'model_eval_data_mastertable_private.csv'
@@ -377,6 +377,52 @@ def upload_model_dict(modelpath, s3_presigned_dict, bucket, model_id, model_vers
                                         'default_value': default_config,
                                         'param_value': model_configvalues})
 
+        elif meta_dict['ml_framework'] in ['pyspark']:
+            import ast
+            import astunparse
+
+            model_config = meta_dict["model_config"]
+            tree = ast.parse(model_config)
+
+            stringconfig=model_config
+
+            problemnodes=[]
+            for node in ast.walk(tree):
+                if isinstance(node, ast.Call):
+                    problemnodes.append(astunparse.unparse(node).replace("\n",""))
+
+            problemnodesunique=set(problemnodes)
+            for i in problemnodesunique:
+                stringconfig=stringconfig.replace(i,"'"+i+"'")
+
+            try:
+                model_config_temp = ast.literal_eval(stringconfig)
+                model_class = pyspark_model_from_string(meta_dict['model_type'])
+                default = model_class()
+
+                # get model config dict from pyspark model object
+                default_config_temp = {}
+                for key, value in default.extractParamMap().items():
+                    default_config_temp[key.name] = value
+                
+                # Sort the keys so default and model config key matches each other
+                model_config = dict(sorted(model_config_temp.items()))
+                default_config = dict(sorted(default_config_temp.items()))
+                
+                model_configkeys = model_config.keys()
+                model_configvalues = model_config.values()
+                default_config = default_config.values()
+            except:
+                model_class = str(pyspark_model_from_string(meta_dict['model_type']))
+                if model_class.find("Voting")>0:
+                      default_config = ["No data available"]
+                      model_configkeys=["No data available"]
+                      model_configvalues=["No data available"]
+
+            inspect_pd = pd.DataFrame({'param_name': model_configkeys,
+                                        'default_value': default_config,
+                                        'param_value': model_configvalues})
+
     else:
         meta_dict = {}
         meta_dict['ml_framework'] = "undefined"
@@ -384,8 +430,6 @@ def upload_model_dict(modelpath, s3_presigned_dict, bucket, model_id, model_vers
 
         #inspect_pd = pd.DataFrame({' ':["No metadata available for this model"]})
         inspect_pd = pd.DataFrame()
-
-
    
     try:
         #Get inspect json
@@ -439,7 +483,7 @@ def upload_model_graph(modelpath,  s3_presigned_dict, bucket, model_id, model_ve
 
         model_graph = ''
 
-    elif meta_dict['ml_framework'] in ['sklearn', 'xgboost']:
+    elif meta_dict['ml_framework'] in ['sklearn', 'xgboost', 'pyspark']:
 
         model_graph = ''
 
@@ -487,7 +531,8 @@ def submit_model(
     prediction_submission=None,
     preprocessor=None,
     reproducibility_env_filepath=None,
-    custom_metadata=None
+    custom_metadata=None,
+    submission_type="competition"
     ):
     """
     Submits model/preprocessor to machine learning competition using live prediction API url generated by AI Modelshare library
@@ -570,6 +615,7 @@ def submit_model(
 
         post_dict = {"y_pred": prediction_submission,
                 "return_eval": "True",
+                "submission_type": submission_type,
                 "return_y": "False"}
 
         headers = { 'Content-Type':'application/json', 'authorizationToken': json.dumps({"token":os.environ.get("AWS_TOKEN"),"eval":"TEST"}), } 
@@ -812,7 +858,6 @@ def submit_model(
             for i in problemnodesunique:
                 stringconfig=stringconfig.replace(i,"'"+i+"'")
 
-
             try:
                 model_config=ast.literal_eval(stringconfig)
                 model_class = model_from_string(meta_dict['model_type'])
@@ -833,14 +878,59 @@ def submit_model(
                                         'param_value': model_configvalues})
 
             model_graph = ''
-        
+
+        elif meta_dict['ml_framework'] in ['pyspark']:
+            import ast
+            import astunparse
+
+            model_config = meta_dict["model_config"]
+            tree = ast.parse(model_config)
+
+            stringconfig=model_config
+
+            problemnodes=[]
+            for node in ast.walk(tree):
+                if isinstance(node, ast.Call):
+                    problemnodes.append(astunparse.unparse(node).replace("\n",""))
+
+            problemnodesunique=set(problemnodes)
+            for i in problemnodesunique:
+                stringconfig=stringconfig.replace(i,"'"+i+"'")
+
+            try:
+                model_config_temp = ast.literal_eval(stringconfig)
+                model_class = pyspark_model_from_string(meta_dict['model_type'])
+                default = model_class()
+
+                # get model config dict from pyspark model object
+                default_config_temp = {}
+                for key, value in default.extractParamMap().items():
+                    default_config_temp[key.name] = value
+                
+                # Sort the keys so default and model config key matches each other
+                model_config = dict(sorted(model_config_temp.items()))
+                default_config = dict(sorted(default_config_temp.items()))
+                
+                model_configkeys = model_config.keys()
+                model_configvalues = model_config.values()
+                default_config = default_config.values()
+            except:
+                model_class = str(pyspark_model_from_string(meta_dict['model_type']))
+                if model_class.find("Voting")>0:
+                      default_config = ["No data available"]
+                      model_configkeys=["No data available"]
+                      model_configvalues=["No data available"]
+
+            inspect_pd = pd.DataFrame({'param_name': model_configkeys,
+                                    'default_value': default_config,
+                                    'param_value': model_configvalues})
+            model_graph = ""
     else: 
 
         inspect_pd = pd.DataFrame()
         model_graph = ''
-
         
-
+        
     keys_to_extract = [ "accuracy", "f1_score", "precision", "recall", "mse", "rmse", "mae", "r2"]
 
     eval_metrics_subset = {key: eval_metrics[key] for key in keys_to_extract}
@@ -859,7 +949,9 @@ def submit_model(
                 "modelsubmissiondescription": modelsubmissiondescription,
                 "modelsubmissiontags":modelsubmissiontags,
                   "eval_metrics":json.dumps(eval_metrics_subset_nonulls),
-                  "eval_metrics_private":json.dumps(eval_metrics_private_subset_nonulls)
+                  "eval_metrics_private":json.dumps(eval_metrics_private_subset_nonulls),
+                  "submission_type": submission_type
+
                   }
 
     bodydatamodels.update(modelleaderboarddata_cleaned)
@@ -888,7 +980,7 @@ def submit_model(
 
     return print("\nYour model has been submitted as model version "+str(model_version)+ "\n\n"+code_comp_result)
 
-def update_runtime_model(apiurl, model_version=None):
+def update_runtime_model(apiurl, model_version=None, submission_type="competition"):
     """
     apiurl: string of API URL that the user wishes to edit
     new_model_version: string of model version number (from leaderboard) to replace original model 
@@ -926,7 +1018,7 @@ def update_runtime_model(apiurl, model_version=None):
 
     try:
         leaderboard = aws_client["client"].get_object(
-            Bucket=api_bucket, Key=model_id + "/model_eval_data_mastertable.csv"
+            Bucket=api_bucket, Key=model_id +"/"+submission_type+"/model_eval_data_mastertable.csv"
 
 
 
@@ -937,23 +1029,25 @@ def update_runtime_model(apiurl, model_version=None):
         leaderboardversion=leaderboard[leaderboard['version']==int(model_version)]
         leaderboardversion=leaderboardversion.dropna(axis=1)
         metric_names_subset=list(set(metric_names).intersection(leaderboardversion.columns))
+        print(metric_names_subset)
         leaderboardversiondict=leaderboardversion.loc[:,metric_names_subset].to_dict('records')[0]
+        print(leaderboardversiondict)
     except Exception as err:
         raise err
 
     # Get file list for current bucket {{{
-    model_files, err = _get_file_list(aws_client, api_bucket, model_id)
+    model_files, err = _get_file_list(aws_client, api_bucket, model_id+"/"+submission_type)
     if err is not None:
         raise err
     # }}}
 
     # extract subfolder objects specific to the model id
-    folder = s3.meta.client.list_objects(Bucket=api_bucket, Prefix=model_id+"/")
+    folder = s3.meta.client.list_objects(Bucket=api_bucket, Prefix=model_id+"/"+submission_type+"/")
     bucket = s3.Bucket(api_bucket)
     file_list = [file['Key'] for file in folder['Contents']]
     s3 = boto3.resource('s3')
-    model_source_key = model_id+"/onnx_model_v"+str(model_version)+".onnx"
-    preprocesor_source_key = model_id+"/preprocessor_v"+str(model_version)+".zip"
+    model_source_key = model_id+"/"+submission_type+"/onnx_model_v"+str(model_version)+".onnx"
+    preprocesor_source_key = model_id+"/"+submission_type+"/preprocessor_v"+str(model_version)+".zip"
     model_copy_source = {
           'Bucket': api_bucket,
           'Key': model_source_key
@@ -973,8 +1067,8 @@ def update_runtime_model(apiurl, model_version=None):
 
     # overwrite runtime_model.onnx file & runtime_preprocessor.zip files: 
     if (model_source_key in file_list) & (preprocesor_source_key in file_list):
-        response = bucket.copy(model_copy_source, model_id+"/"+'runtime_model.onnx')
-        response = bucket.copy(preprocessor_copy_source, model_id+"/"+'runtime_preprocessor.zip')
+        response = bucket.copy(model_copy_source, model_id+"/"+submission_type+"/"+'runtime_model.onnx')
+        response = bucket.copy(preprocessor_copy_source, model_id+"/"+submission_type+"/"+'runtime_preprocessor.zip')
         return print('Runtime model & preprocessor for api: '+apiurl+" updated to model version "+model_version+".\n\nModel metrics are now updated and verified for this model playground.")
     else:
         # the file resource to be the new runtime_model is not available
